@@ -326,7 +326,7 @@ anjuta_plugin_class_init (AnjutaPluginClass *class)
 				      _("Anjuta Shell"),
 				      _("Anjuta shell that will contain the plugin"),
 				      ANJUTA_TYPE_SHELL,
-				      G_PARAM_READWRITE/* | G_PARAM_CONSTRUCT_ONLY*/)); /* Construct only does not work with libanjutamm */
+				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 	plugin_signals[ACTIVATED_SIGNAL] =
 		g_signal_new ("activated",
 					G_TYPE_FROM_CLASS (object_class),
@@ -405,8 +405,8 @@ value_removed_cb (AnjutaShell *shell,
  * anjuta_plugin_add_watch:
  * @plugin: a #AnjutaPlugin derived class object.
  * @name: Name of the value to watch.
- * @added: Callback to call when the value is added.
- * @removed: Callback to call when the value is removed.
+ * @added: (closure user_data) (scope call): Callback to call when the value is added.
+ * @removed: (closure user_data) (scope call): Callback to call when the value is removed.
  * @user_data: User data to pass to callbacks.
  * 
  * Adds a watch for @name value. When the value is added in shell, the @added
