@@ -54,8 +54,8 @@ struct _AnjutaBookmarksPrivate
 	GtkWidget* button_remove;
 	GtkWidget* grip;
 
+	IAnjutaSymbolQuery* query_scope;
 	GtkWidget* menu;
-	IAnjutaSymbolQuery *query_scope;
 	DocmanPlugin* docman;
 };
 
@@ -70,6 +70,9 @@ enum
 
 G_DEFINE_TYPE (AnjutaBookmarks, anjuta_bookmarks, G_TYPE_OBJECT);
 
+static gchar*
+anjuta_bookmarks_get_text (AnjutaBookmarks* bookmarks, IAnjutaEditor* editor, gint line, gboolean use_selection);
+
 static void
 on_document_changed (AnjutaDocman *docman, IAnjutaDocument *doc,
 					 AnjutaBookmarks *bookmarks)
@@ -136,14 +139,34 @@ on_row_activate (GtkTreeView* view, GtkTreePath* path,
 	GtkTreeIter iter;
 	GFile* file;
 	gint line;
+	gint handle;
 	AnjutaBookmarksPrivate* priv = BOOKMARKS_GET_PRIVATE(bookmarks);
 	IAnjutaEditor* editor;
 	gtk_tree_model_get_iter (priv->model, &iter, path);
 	gtk_tree_model_get (priv->model, &iter,
 						COLUMN_FILE, &file,
 						COLUMN_LINE, &line,
+	                    COLUMN_HANDLE, &handle,
 						-1);
+	
 	editor = anjuta_docman_goto_file_line (ANJUTA_DOCMAN(priv->docman->docman), file, line);
+	if (editor)
+	{
+		line = ianjuta_markable_location_from_handle (IANJUTA_MARKABLE (editor),
+		                                                        handle, NULL);
+		if (line >= 0)
+		{
+			gchar* new_title = anjuta_bookmarks_get_text (bookmarks,
+			                                              editor,
+			                                              line,
+			                                              FALSE);
+			anjuta_docman_goto_file_line (ANJUTA_DOCMAN(priv->docman->docman), file, line);
+			gtk_list_store_set (GTK_LIST_STORE (priv->model), &iter,
+			                    COLUMN_TEXT, new_title,
+			                    COLUMN_LINE, line, -1);
+			g_free (new_title);
+		}
+	}
 	g_object_unref (file);
 }
 
@@ -222,7 +245,7 @@ on_document_added (AnjutaDocman* docman, IAnjutaDocument* doc,
 			{
 				int handle = ianjuta_markable_mark (markable, line,
 												   IANJUTA_MARKABLE_BOOKMARK,
-												   NULL);
+												   NULL, NULL);
 				gtk_list_store_set (GTK_LIST_STORE(priv->model),
 									&iter,
 									COLUMN_HANDLE,
@@ -301,7 +324,7 @@ anjuta_bookmarks_init (AnjutaBookmarks *bookmarks)
 					  bookmarks);
 	
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(priv->tree));
-	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
+	gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
 	g_signal_connect (G_OBJECT(selection), "changed", G_CALLBACK(on_selection_changed),
 					  bookmarks);
 	
@@ -356,8 +379,7 @@ anjuta_bookmarks_finalize (GObject *object)
 {
 	AnjutaBookmarks* bookmarks = ANJUTA_BOOKMARKS (object);
 	AnjutaBookmarksPrivate* priv = BOOKMARKS_GET_PRIVATE(bookmarks);
-
-	g_object_unref (priv->query_scope);
+	
 	gtk_widget_destroy (priv->menu);
 	
 	anjuta_shell_remove_widget (ANJUTA_PLUGIN(priv->docman)->shell,
@@ -386,18 +408,6 @@ anjuta_bookmarks_new (DocmanPlugin* docman)
 	AnjutaBookmarksPrivate* priv = BOOKMARKS_GET_PRIVATE(bookmarks);
 	priv->docman = docman;
 
-	IAnjutaSymbolManager* sym_manager;
-	static IAnjutaSymbolField fields[] = {IANJUTA_SYMBOL_FIELD_NAME};
-	
-	sym_manager = anjuta_shell_get_interface (ANJUTA_PLUGIN(priv->docman)->shell,
-											  IAnjutaSymbolManager, NULL);
-	priv->query_scope =
-		ianjuta_symbol_manager_create_query (sym_manager,
-											 IANJUTA_SYMBOL_QUERY_SEARCH_SCOPE,
-											  IANJUTA_SYMBOL_QUERY_DB_PROJECT,
-											  NULL);
-	ianjuta_symbol_query_set_fields (priv->query_scope, 1, fields, NULL);
-	
 	anjuta_shell_add_widget_custom (ANJUTA_PLUGIN(docman)->shell,
 	                                priv->window,
 	                                "bookmarks",
@@ -414,12 +424,33 @@ anjuta_bookmarks_new (DocmanPlugin* docman)
 	return bookmarks;
 }
 
+static IAnjutaSymbolQuery*
+anjuta_bookmarks_create_query (AnjutaBookmarks* bookmarks)
+{
+	AnjutaBookmarksPrivate* priv = BOOKMARKS_GET_PRIVATE(bookmarks);
+	IAnjutaSymbolManager* sym_manager;
+	static IAnjutaSymbolField fields[] = {IANJUTA_SYMBOL_FIELD_NAME};
+	
+	sym_manager = anjuta_shell_get_interface (ANJUTA_PLUGIN(priv->docman)->shell,
+											  IAnjutaSymbolManager, NULL);
+	
+	IAnjutaSymbolQuery * query_scope =
+		ianjuta_symbol_manager_create_query (sym_manager,
+											 IANJUTA_SYMBOL_QUERY_SEARCH_SCOPE,
+											  IANJUTA_SYMBOL_QUERY_DB_PROJECT,
+											  NULL);
+	ianjuta_symbol_query_set_fields (query_scope, 1, fields, NULL);
+	
+	return query_scope;
+}
+
 static gchar*
 anjuta_bookmarks_get_text_from_file (AnjutaBookmarks* bookmarks, GFile* file, gint line)
 {
-	gchar* text;
 	AnjutaBookmarksPrivate* priv = BOOKMARKS_GET_PRIVATE(bookmarks);
-
+	gchar* text;
+	
+	
 	if (priv->query_scope != NULL)
 	{
 		gchar* path = g_file_get_path (file);
@@ -429,10 +460,13 @@ anjuta_bookmarks_get_text_from_file (AnjutaBookmarks* bookmarks, GFile* file, gi
 		g_free (path);
 		if (iter)
 		{
-			text = g_strdup (ianjuta_symbol_get_string (IANJUTA_SYMBOL(iter),
-														IANJUTA_SYMBOL_FIELD_NAME,
-														NULL));
-			g_object_unref (iter);
+			const gchar* symbol_name;
+			symbol_name = ianjuta_symbol_get_string (IANJUTA_SYMBOL(iter),
+			                                         IANJUTA_SYMBOL_FIELD_NAME,
+			                                         NULL);
+			text = g_strdup_printf ("%s:%d", symbol_name,
+			                        line);
+			g_object_unref (iter);			
 			return text;
 		}
 	}
@@ -476,6 +510,26 @@ anjuta_bookmarks_get_text (AnjutaBookmarks* bookmarks, IAnjutaEditor* editor, gi
 	}								  
 }
 
+static void
+anjuta_bookmarks_check_selection (AnjutaBookmarks* bookmarks,
+                                  GtkTreePath* path)
+{
+	AnjutaBookmarksPrivate* priv = BOOKMARKS_GET_PRIVATE(bookmarks);
+	GtkTreeSelection* selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(priv->tree));
+	GList* selected = gtk_tree_selection_get_selected_rows (selection,
+															NULL);
+	if (!selected)
+	{
+		GtkTreeIter iter;
+		if (!path || !gtk_tree_model_get_iter (priv->model, &iter, path))
+		{
+			if (!gtk_tree_model_get_iter_first (priv->model, &iter))
+				return;
+		}
+		gtk_tree_selection_select_iter (selection, &iter);
+	}	
+}
+
 void
 anjuta_bookmarks_add (AnjutaBookmarks* bookmarks, IAnjutaEditor* editor, gint line, 
 					  const gchar* title, gboolean use_selection)
@@ -492,7 +546,7 @@ anjuta_bookmarks_add (AnjutaBookmarks* bookmarks, IAnjutaEditor* editor, gint li
 	if (ianjuta_markable_is_marker_set (markable, line, IANJUTA_MARKABLE_BOOKMARK, NULL))
 		return;
 	
-	handle = ianjuta_markable_mark (markable, line, IANJUTA_MARKABLE_BOOKMARK, NULL);
+	handle = ianjuta_markable_mark (markable, line, IANJUTA_MARKABLE_BOOKMARK, NULL, NULL);
 	
 	gtk_list_store_append (GTK_LIST_STORE(priv->model), &iter);
 	if (title == NULL)
@@ -522,6 +576,7 @@ anjuta_bookmarks_remove (AnjutaBookmarks* bookmarks)
 															NULL);
 	GList* node;
 	GList* refs = NULL;
+	GtkTreePath* first_path = NULL;
 	for (node = selected; node != NULL; node = g_list_next (node))
 	{
 		GtkTreeRowReference* ref = gtk_tree_row_reference_new (priv->model,
@@ -541,7 +596,14 @@ anjuta_bookmarks_remove (AnjutaBookmarks* bookmarks)
 		gtk_tree_model_get_iter (priv->model,
 								 &iter,
 								 path);
-		gtk_tree_path_free (path);
+		if (!first_path)
+		{
+			first_path = path;
+		}
+		else
+		{
+			gtk_tree_path_free (path);
+		}
 		gtk_tree_model_get (priv->model, &iter, 
 							COLUMN_FILE, &file,
 							COLUMN_LINE, &line,
@@ -561,6 +623,11 @@ anjuta_bookmarks_remove (AnjutaBookmarks* bookmarks)
 		
 		gtk_list_store_remove (GTK_LIST_STORE (priv->model), &iter);
 	}
+
+	anjuta_bookmarks_check_selection (bookmarks, first_path);
+	if (first_path)
+		gtk_tree_path_free (first_path);
+	
 	g_list_foreach (refs, (GFunc)gtk_tree_row_reference_free, NULL);
 	g_list_free (refs);
 }
@@ -714,6 +781,54 @@ anjuta_bookmarks_session_save (AnjutaBookmarks* bookmarks, AnjutaSession* sessio
 	
 	/* Clear the model */
 	gtk_list_store_clear (GTK_LIST_STORE (priv->model));
+
+	if (priv->query_scope)
+		g_object_unref (priv->query_scope);
+}
+
+void 
+anjuta_bookmarks_toggle (AnjutaBookmarks* bookmarks, 
+                         IAnjutaEditor* editor, 
+                         gint line)
+{
+	AnjutaBookmarksPrivate* priv = BOOKMARKS_GET_PRIVATE(bookmarks);
+	
+	g_return_if_fail (bookmarks != NULL);
+	g_return_if_fail (editor != NULL);
+
+	/* Check if there is a bookmark in that line already */
+	if (ianjuta_markable_is_marker_set (IANJUTA_MARKABLE (editor),
+	                                    line, IANJUTA_MARKABLE_BOOKMARK, NULL))
+	{
+		GtkTreeIter iter;
+		if (gtk_tree_model_get_iter_first (priv->model, &iter))
+		{
+			do
+			{
+				gint handle;
+				gint location;
+				gtk_tree_model_get (priv->model, &iter,
+				                    COLUMN_HANDLE, &handle, -1);
+
+				/* Update location if necessary */
+				location = ianjuta_markable_location_from_handle (IANJUTA_MARKABLE (editor),
+				                                                  handle, NULL);
+				gtk_list_store_set (GTK_LIST_STORE (priv->model), &iter,
+					                    COLUMN_LINE, location, -1 );
+				if (line == location)
+				{
+					GtkTreeSelection* selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->tree));
+					gtk_tree_selection_select_iter (selection, &iter);
+					anjuta_bookmarks_remove (bookmarks);
+				}
+			}
+			while (gtk_tree_model_iter_next (priv->model, &iter));
+		}
+	}
+	else
+	{
+		anjuta_bookmarks_add (bookmarks, editor, line, NULL, TRUE);
+	}
 }
 
 static void
@@ -751,6 +866,7 @@ read_bookmarks (AnjutaBookmarks* bookmarks, xmlNodePtr marks)
 void
 anjuta_bookmarks_session_load (AnjutaBookmarks* bookmarks, AnjutaSession* session)
 {
+	AnjutaBookmarksPrivate* priv = BOOKMARKS_GET_PRIVATE(bookmarks);	
 	gchar* xml_string = anjuta_session_get_string (session,
 												   "Document Manager",
 												   "bookmarks");
@@ -774,6 +890,8 @@ anjuta_bookmarks_session_load (AnjutaBookmarks* bookmarks, AnjutaSession* sessio
 		read_bookmarks (bookmarks, cur->children);
 	
 	xmlFreeDoc (doc);
+
+	priv->query_scope = anjuta_bookmarks_create_query (bookmarks);
 }
 
 static gint
