@@ -43,10 +43,17 @@ struct _AnjutaDesignDocumentPrivate
 #define ADD_GET_PRIVATE(o) \
 	(G_TYPE_INSTANCE_GET_PRIVATE ((o), ANJUTA_TYPE_DESIGN_DOCUMENT, AnjutaDesignDocumentPrivate))
 
+static void
+on_project_changed (GladeProject* project, GladeCommand* command, gboolean forward, AnjutaDesignDocument* doc)
+{
+	g_signal_emit_by_name (doc, "update-save-ui", NULL);
+	g_signal_emit_by_name (doc, "update-ui", NULL);
+}
+
 static void
 anjuta_design_document_instance_init (AnjutaDesignDocument *object)
 {
-	
+
 }
 
 static void
@@ -102,10 +109,13 @@ anjuta_design_document_class_init (AnjutaDesignDocumentClass *klass)
 GtkWidget*
 anjuta_design_document_new (GladePlugin* glade_plugin, GladeProject* project)
 {
-	return GTK_WIDGET(g_object_new(ANJUTA_TYPE_DESIGN_DOCUMENT, 
-								   "plugin", glade_plugin,
-								   "project", project,
-								   NULL));
+	GObject* doc = g_object_new(ANJUTA_TYPE_DESIGN_DOCUMENT, 
+	                            "plugin", glade_plugin,
+	                            "project", project,
+	                            NULL);
+	g_signal_connect (project, "changed", G_CALLBACK (on_project_changed), doc);
+
+	return GTK_WIDGET (doc);
 }
 
 static void ifile_open(IAnjutaFile* ifile, GFile* file, GError **e)
@@ -243,7 +253,6 @@ static gboolean idocument_can_redo(IAnjutaDocument *editor, GError **e)
 {
 	AnjutaDesignDocument* self = ANJUTA_DESIGN_DOCUMENT(editor);
 	GladeCommand *redo_item;
-	const gchar *redo_description = NULL;
 
 	GladeProject* project = glade_design_view_get_project(GLADE_DESIGN_VIEW(self));
 	if (!project)
@@ -253,10 +262,8 @@ static gboolean idocument_can_redo(IAnjutaDocument *editor, GError **e)
 	else
 	{
 		redo_item = glade_project_next_redo_item(project);
-		if (redo_item)
-			redo_description = redo_item->description;
 	}
-	return (redo_description != NULL);
+	return (redo_item != NULL);
 }
 
 /* Return true if editor can undo */
@@ -264,7 +271,6 @@ static gboolean idocument_can_undo(IAnjutaDocument *editor, GError **e)
 {
 	AnjutaDesignDocument* self = ANJUTA_DESIGN_DOCUMENT(editor);
 	GladeCommand *undo_item;
-	const gchar *undo_description = NULL;
 	GladeProject* project = glade_design_view_get_project(GLADE_DESIGN_VIEW(self));
 	if (!project)
 	{
@@ -273,10 +279,9 @@ static gboolean idocument_can_undo(IAnjutaDocument *editor, GError **e)
 	else
 	{
 		undo_item = glade_project_next_undo_item(project);
-		if (undo_item)
-			undo_description = undo_item->description;
+
 	}
-	return (undo_description != NULL);
+	return (undo_item != NULL);
 }
 
 /* Return true if editor can undo */
@@ -295,13 +300,15 @@ static void idocument_end_undo_action (IAnjutaDocument *editor, GError **e)
 static void 
 idocument_undo(IAnjutaDocument* edit, GError** ee)
 {
-	glade_app_command_undo();
+	GladeDesignView* view = GLADE_DESIGN_VIEW (edit);
+	glade_project_undo (glade_design_view_get_project (view));
 }
 
 static void 
 idocument_redo(IAnjutaDocument* edit, GError** ee)
 {
-	glade_app_command_redo();
+	GladeDesignView* view = GLADE_DESIGN_VIEW (edit);
+	glade_project_undo (glade_design_view_get_project (view));
 }
 
 /* Grab focus */
@@ -321,25 +328,30 @@ static const gchar* idocument_get_filename(IAnjutaDocument *editor, GError **e)
 static void 
 idocument_cut(IAnjutaDocument* edit, GError** ee)
 {
-	glade_app_command_cut();
+	GladeDesignView* view = GLADE_DESIGN_VIEW (edit);
+	glade_project_command_cut (glade_design_view_get_project (view));
 }
 
 static void 
 idocument_copy(IAnjutaDocument* edit, GError** ee)
 {
- 	glade_app_command_copy();
+	GladeDesignView* view = GLADE_DESIGN_VIEW (edit);
+	glade_project_copy_selection (glade_design_view_get_project (view));
 }
 
 static void 
 idocument_paste(IAnjutaDocument* edit, GError** ee)
 {
-	glade_app_command_paste(NULL);
+	GladeDesignView* view = GLADE_DESIGN_VIEW (edit);
+	glade_project_command_paste (glade_design_view_get_project (view), NULL);
 }
 
 static void 
 idocument_clear(IAnjutaDocument* edit, GError** ee)
 {
-	glade_app_command_delete();
+	GladeDesignView* view = GLADE_DESIGN_VIEW (edit);
+	glade_project_command_delete (glade_design_view_get_project (view));
+
 }
 
 
