@@ -45,7 +45,13 @@
 typedef struct _PropertiesTable
 {
 	AnjutaPmProject *project;
+	GtkWidget *dialog;
 	GtkWidget *table;
+	GtkWidget *head;
+	GtkWidget *main;
+	GtkWidget *expand;
+	GtkWidget *extra;
+	GbfTreeData *data;
 	AnjutaProjectNode *node;
 	GList *properties;
 } PropertiesTable;
@@ -201,14 +207,14 @@ module_filter_func (GtkTreeModel *model, GtkTreeIter *iter, gpointer user_data)
 	GbfTreeData *data = NULL;
 	gboolean visible = FALSE;
 	AnjutaProjectNode *node;
-
+	
 	gtk_tree_model_get (model, iter,
 						GBF_PROJECT_MODEL_COLUMN_DATA, &data, -1);
 	node = data == NULL ? NULL : gbf_tree_data_get_node (data);
 	if (node != NULL)
 	{
 		AnjutaProjectNodeType type = anjuta_project_node_get_node_type (node);
-		
+
 		visible = (type == ANJUTA_PROJECT_MODULE) || (type == ANJUTA_PROJECT_PACKAGE);
 	}
 	
@@ -216,57 +222,19 @@ module_filter_func (GtkTreeModel *model, GtkTreeIter *iter, gpointer user_data)
 }
 
 static void 
-setup_nodes_treeview (GbfProjectModel         *model,
-						GtkWidget              *view,
+setup_nodes_treeview (GbfProjectView           *view,
+						GbfProjectView		   *parent,
+                  	    GtkTreePath            *root,
 						GtkTreeModelFilterVisibleFunc func,
 						gpointer               data,
 						GtkTreeIter            *selected)
 {
-	GtkTreeModel *filter;
-	GtkTreeIter iter_filter;
-	GtkTreePath *path;
-
-	g_return_if_fail (model != NULL);
 	g_return_if_fail (view != NULL && GBF_IS_PROJECT_VIEW (view));
+	g_return_if_fail (parent != NULL);
 
-	filter = gtk_tree_model_filter_new (GTK_TREE_MODEL (model), NULL);
-	gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (filter),
-											func, data, NULL);
-	gtk_tree_view_set_model (GTK_TREE_VIEW (view), GTK_TREE_MODEL (filter));
-	g_object_unref (filter);
-
-	/* select default node */
-	if (selected && gtk_tree_model_filter_convert_child_iter_to_iter (
-			GTK_TREE_MODEL_FILTER (filter), &iter_filter, selected))
-	{
-		path = gtk_tree_model_get_path (filter, &iter_filter);
-		gtk_tree_view_expand_to_path (GTK_TREE_VIEW (view), path);
-	}
-	else
-	{
-		GtkTreePath *root_path;
-
-		gtk_tree_view_expand_all (GTK_TREE_VIEW (view));
-		root_path = gbf_project_model_get_project_root (model);
-		if (root_path)
-		{
-			path = gtk_tree_model_filter_convert_child_path_to_path (
-						GTK_TREE_MODEL_FILTER (filter), root_path);
-			gtk_tree_path_free (root_path);
-		}
-		else
-		{
-			path = gtk_tree_path_new_first ();
-		}
-	}
-
-	if (path)
-	{
-		gtk_tree_view_set_cursor (GTK_TREE_VIEW (view), path, NULL, FALSE);
-		gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (view), path, NULL,
-									TRUE, 0.5, 0.0);
-		gtk_tree_path_free (path);
-	}
+	gbf_project_view_set_parent_view (view, parent, root);
+	gbf_project_view_set_visible_func (view, func, data, NULL);
+	gbf_project_view_set_cursor_to_iter (view, selected);
 }
 
 static void
@@ -383,6 +351,7 @@ add_entry (IAnjutaProject *project, AnjutaProjectNode *node, AnjutaProjectProper
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	GList *item;
+	gchar *tooltip = NULL;
 
 	if (prop->native != NULL)
 	{
@@ -392,6 +361,23 @@ add_entry (IAnjutaProject *project, AnjutaProjectNode *node, AnjutaProjectProper
 	{
 		label = gtk_label_new (_(prop->name));
 	}
+
+	if (prop->detail != NULL)
+	{
+		if (prop->flags & ANJUTA_PROJECT_PROPERTY_READ_ONLY)
+		{
+			tooltip = g_strconcat (_(prop->detail), _(" This property is not modifiable."), NULL);
+		}
+		else
+		{
+			tooltip = g_strdup (_(prop->detail));
+		}
+	}
+	
+	if (tooltip != NULL)
+	{
+		gtk_widget_set_tooltip_markup (label, tooltip);
+	}
 	gtk_misc_set_alignment (GTK_MISC (label), 0, -1);
 	gtk_widget_show (label);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, *position, *position+1,
@@ -458,6 +444,12 @@ add_entry (IAnjutaProject *project, AnjutaProjectNode *node, AnjutaProjectProper
 	default:
 		return NULL;
 	}		
+	if (tooltip != NULL)
+	{
+		gtk_widget_set_tooltip_markup (entry, tooltip);
+	}
+	g_free (tooltip);
+	
 	gtk_widget_show (entry);
 	gtk_table_attach (GTK_TABLE (table), entry, 1, 2, *position, *position+1,
 			  GTK_FILL | GTK_EXPAND, GTK_FILL, 5, 3);
@@ -487,60 +479,100 @@ add_label (const gchar *display_name, const gchar *value, GtkWidget *table, gint
 	*position = *position + 1;
 }
 
-static PropertiesTable*
-create_properties_table (IAnjutaProject *project, AnjutaProjectNode *node)
+static gboolean
+is_project_node_but_shortcut (GtkTreeModel *model, GtkTreeIter *iter, gpointer user_data)
 {
-	GtkBuilder *bxml;
-	PropertiesTable *table;
-	GtkWidget *properties;
-	GtkWidget *main_table;
-	GtkWidget *extra_table;
-	GtkWidget *extra_expand;
-	GFile *file;
+	GbfTreeData *data;
 
+	gtk_tree_model_get (GTK_TREE_MODEL (model), iter,
+		    GBF_PROJECT_MODEL_COLUMN_DATA, &data,
+		    -1);
+
+	return (data != NULL) && (data->shortcut == NULL) && (gbf_tree_data_get_node (data) != NULL);
+}
+
+static void
+update_properties (PropertiesTable *table)
+{
+	GFile *file;
+	const gchar *title;
+	gint head_pos;
 	gint main_pos;
 	gint extra_pos;
-	gchar *path;
 	const GList *item;
 	AnjutaProjectNodeType type;
 	AnjutaProjectNodeInfo* node_info;
 	gboolean single;
+	GList *children;
+	GList *last;
 
-	bxml = anjuta_util_builder_new (GLADE_FILE, NULL);
-	if (!bxml) return NULL;
-
-	table = g_new0 (PropertiesTable, 1);
-	table->node = node;
-	anjuta_util_builder_get_objects (bxml,
-									"properties", &properties,
-									"main_table", &main_table,
-									"extra_table", &extra_table,
-									"extra_expand", &extra_expand,
-									NULL);
-	table->table = properties;
-	g_object_ref (properties);
-
+	head_pos = 0;
 	main_pos = 0;
 	extra_pos = 0;
 
-	/* Always display node name */
-	file = anjuta_project_node_get_file (node);
+	/* Update dialog type */
+	switch (anjuta_project_node_get_node_type (table->node))
+	{
+	case ANJUTA_PROJECT_ROOT:
+		title = _("Project properties");
+		break;
+	case ANJUTA_PROJECT_GROUP:
+		title = _("Directory properties");
+		break;
+	case ANJUTA_PROJECT_TARGET:
+		title = _("Target properties");
+		break;
+	case ANJUTA_PROJECT_SOURCE:
+		title = _("Source properties");
+		break;
+	case ANJUTA_PROJECT_MODULE:
+		title = _("Module properties");
+		break;
+	case ANJUTA_PROJECT_PACKAGE:
+		title = _("Package properties");
+		break;
+	default:
+		title = _("Unknown properties");
+		break;
+	}
+	gtk_window_set_title (GTK_WINDOW (table->dialog), title);
+
+	/* Clear table */
+	children = gtk_container_get_children (GTK_CONTAINER (table->head));
+	/* Clear only the first 4 widgets */
+	while ((last = g_list_nth (children, 4)) != NULL) children = g_list_delete_link (children, last);
+	g_list_foreach (children, (GFunc)gtk_widget_destroy, NULL);
+	g_list_free (children);
+	children = gtk_container_get_children (GTK_CONTAINER (table->main));
+	g_list_foreach (children, (GFunc)gtk_widget_destroy, NULL);
+	g_list_free (children);
+	children = gtk_container_get_children (GTK_CONTAINER (table->extra));
+	g_list_foreach (children, (GFunc)gtk_widget_destroy, NULL);
+	g_list_free (children);
+	g_list_foreach (table->properties, (GFunc)pm_property_entry_free, NULL);
+	g_list_free (table->properties);
+	table->properties = NULL;
+	
+	/* Update node name */
+	file = anjuta_project_node_get_file (table->node);
 	if (file != NULL)
 	{
+		gchar *path;
+		
 		path = g_file_get_path (file);
-		add_label (_("Path:"), path, main_table, &main_pos);
+		add_label (_("Path:"), path, table->head, &head_pos);
 		g_free (path);
 	}
 	else
 	{
-		add_label (_("Name:"), anjuta_project_node_get_name (node), main_table, &main_pos);
+		add_label (_("Name:"), anjuta_project_node_get_name (table->node), table->head, &head_pos);
 	}
-
+	
 	/* Display node type only if several types are possible */
 	node_info = NULL;
 	single = TRUE;
-	type = anjuta_project_node_get_full_type (node);
-	for (item = ianjuta_project_get_node_info (project, NULL); item != NULL; item = g_list_next (item))
+	type = anjuta_project_node_get_full_type (table->node);
+	for (item = ianjuta_project_get_node_info (table->project->project, NULL); item != NULL; item = g_list_next (item))
 	{
 		AnjutaProjectNodeInfo* info = (AnjutaProjectNodeInfo *)item->data;
 
@@ -555,28 +587,28 @@ create_properties_table (IAnjutaProject *project, AnjutaProjectNode *node)
 	}
 	if (!single && (node_info != NULL))
 	{
-		add_label (_("Type:"), anjuta_project_node_info_name (node_info), main_table, &main_pos);
+		add_label (_("Type:"), anjuta_project_node_info_name (node_info), table->main, &main_pos);
 	}
 
 	/* Display other node properties */
 	single = FALSE;
 
-	for (item = anjuta_project_node_get_native_properties (node); item != NULL; item = g_list_next (item))
+	for (item = anjuta_project_node_get_native_properties (table->node); item != NULL; item = g_list_next (item))
 	{
 		AnjutaProjectProperty *valid_prop = (AnjutaProjectProperty *)item->data; 
 		AnjutaProjectProperty *prop; 
 		GtkWidget *entry;
 
-		prop = anjuta_project_node_get_property (node, valid_prop);
-		if (prop != NULL)
+		prop = anjuta_project_node_get_property (table->node, valid_prop);
+		if (prop->native != NULL)
 		{
 			/* This property has been set, display it in the main part */
-			entry = add_entry (project, node, prop, main_table, &main_pos);
+			entry = add_entry (table->project->project, table->node, prop, table->main, &main_pos);
 		}
 		else
 		{
 			/* This property has not been set, hide it by default */
-			entry = add_entry (project, node, valid_prop, extra_table, &extra_pos);
+			entry = add_entry (table->project->project, table->node, valid_prop, table->extra, &extra_pos);
 			single = TRUE;
 		}
 
@@ -587,14 +619,13 @@ create_properties_table (IAnjutaProject *project, AnjutaProjectNode *node)
 		}
 	}
 	table->properties = g_list_reverse (table->properties);
-	gtk_widget_show_all (properties);
+	gtk_widget_show_all (table->table);
 	
 	/* Hide expander if it is empty */
-	if (!single) gtk_widget_hide (extra_expand);
-	
-	g_object_unref (bxml);
-	
-	return table;
+	if (single)
+		gtk_widget_show (table->expand);
+	else
+		gtk_widget_hide (table->expand);
 }
 
 static void
@@ -662,68 +693,167 @@ on_properties_dialog_response (GtkWidget *dialog,
 	gtk_widget_destroy (dialog);
 }
 
+static void
+on_node_changed (GtkTreeView       *view,
+                 gpointer           user_data)     
+{
+	PropertiesTable *table = (PropertiesTable *)user_data;
+	GtkTreeIter iter;
+	GtkTreePath *path;
+	GtkTreeModel *model;
+
+	model = gtk_tree_view_get_model (view);
+	gtk_tree_view_get_cursor (view, &path, NULL);
+	
+	if (gtk_tree_model_get_iter (model, &iter, path))
+	{
+		GbfTreeData *data;
+		
+		gtk_tree_model_get (model, &iter, GBF_PROJECT_MODEL_COLUMN_DATA, &data, -1);
+
+		if (table->data->properties_dialog != NULL)
+		{
+			g_object_remove_weak_pointer (G_OBJECT (table->dialog), (gpointer *)&table->data->properties_dialog);
+			table->data->properties_dialog = NULL;
+		}
+		if (data->properties_dialog != NULL)
+		{
+			g_object_unref (data->properties_dialog);
+		}
+		table->data = data;
+		data->properties_dialog = table->dialog;
+		g_object_add_weak_pointer (G_OBJECT (table->dialog), (gpointer *)&table->data->properties_dialog);
+
+		table->node = gbf_tree_data_get_node (data);
+		update_properties (table);
+	}
+
+	if (path != NULL) gtk_tree_path_free (path);
+}
+
+static GtkWidget *
+pm_project_create_properties_dialog (AnjutaPmProject *project, GtkWindow *parent, GbfProjectView *view, GbfTreeData *data, GtkTreeIter *selected)
+{
+	PropertiesTable *table;
+	GtkWidget *dialog = NULL;
+	GtkBuilder *bxml;
+	GtkWidget *node_view;
+	
+	g_return_val_if_fail (data != NULL, NULL);
+	
+	bxml = anjuta_util_builder_new (GLADE_FILE, NULL);
+	if (!bxml) return NULL;
+
+	table = g_new0 (PropertiesTable, 1);
+	table->data = data;
+	table->node = gbf_tree_data_get_node (data);
+	table->project = project;
+	anjuta_util_builder_get_objects (bxml,
+									"properties", &table->table,
+									"head_table", &table->head,
+	                                "nodes_view", &node_view,
+									"main_table", &table->main,
+									"extra_table", &table->extra,
+									"extra_expand", &table->expand,
+									NULL);
+	g_object_ref (table->table);
+	g_object_unref (bxml);
+
+	/* Add tree view */
+	setup_nodes_treeview (GBF_PROJECT_VIEW (node_view),
+	                        view,
+	                        NULL,
+							is_project_node_but_shortcut,
+							NULL,
+							selected);
+	gtk_widget_show (node_view);
+
+	dialog = gtk_dialog_new_with_buttons (NULL,
+						   parent,
+						   GTK_DIALOG_DESTROY_WITH_PARENT,
+						   GTK_STOCK_CANCEL,
+						   GTK_RESPONSE_CANCEL,
+						   GTK_STOCK_APPLY,
+						   GTK_RESPONSE_APPLY, NULL);
+	table->dialog = dialog;
+
+	update_properties (table);
+	
+	g_signal_connect (node_view, "cursor-changed",
+					G_CALLBACK (on_node_changed),
+					table);
+	
+	g_signal_connect (dialog, "response",
+					G_CALLBACK (on_properties_dialog_response),
+					table);
+
+	gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG(dialog))),
+					table->table);
+	gtk_window_set_default_size (GTK_WINDOW (dialog), 450, -1);
+	gtk_widget_show (dialog);
+	
+	return dialog;
+}
+
+
 /* Properties dialog
  *---------------------------------------------------------------------------*/
 
-GtkWidget *
-pm_project_create_properties_dialog (AnjutaPmProject *project, GtkWindow *parent, AnjutaProjectNode *node)
+/* Display properties dialog. These dialogs are not modal, so a pointer on each
+ * dialog is kept with in node data to be able to destroy them if the node is
+ * removed. It is useful to put the dialog at the top if the same target is
+ * selected while the corresponding dialog already exist instead of creating
+ * two times the same dialog.
+ * The project properties dialog is display if the node iterator is NULL. */
+
+gboolean
+anjuta_pm_project_show_properties_dialog (ProjectManagerPlugin *plugin, GtkTreeIter *selected)
 {
-	const char *title;
-	PropertiesTable *table;
-	GtkWidget *dialog = NULL;
-
-	g_return_val_if_fail (node != NULL, NULL);
+	GtkWidget **dialog_ptr;
+	GtkTreeIter iter;
 	
-	switch (anjuta_project_node_get_node_type (node))
+	if (selected == NULL)
 	{
-	case ANJUTA_PROJECT_ROOT:
-		title = _("Project properties");
-		break;
-	case ANJUTA_PROJECT_GROUP:
-		title = _("Group properties");
-		break;
-	case ANJUTA_PROJECT_TARGET:
-		title = _("Target properties");
-		break;
-	case ANJUTA_PROJECT_SOURCE:
-		title = _("Source properties");
-		break;
-	case ANJUTA_PROJECT_MODULE:
-		title = _("Module properties");
-		break;
-	case ANJUTA_PROJECT_PACKAGE:
-		title = _("Package properties");
-		break;
-	default:
-		return NULL;
+		/* Display root properties by default */
+		if (gbf_project_view_get_project_root (plugin->view, &iter))
+		{
+			selected = &iter;
+		}
 	}
 
-	table = create_properties_table (project->project, node);
-
-	if (table != NULL)
+	if (selected)
 	{
-		table->project = project;
-		dialog = gtk_dialog_new_with_buttons (title,
-							   parent,
-							   GTK_DIALOG_DESTROY_WITH_PARENT,
-							   GTK_STOCK_CANCEL,
-							   GTK_RESPONSE_CANCEL,
-							   GTK_STOCK_APPLY,
-							   GTK_RESPONSE_APPLY, NULL);
+		GbfTreeData *data;
+		
+		gtk_tree_model_get (GTK_TREE_MODEL (gbf_project_view_get_model (plugin->view)), selected, GBF_PROJECT_MODEL_COLUMN_DATA, &data, -1);
 
-		g_signal_connect (dialog, "response",
-						G_CALLBACK (on_properties_dialog_response),
-						table);
-
-		gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG(dialog))),
-				table->table);
-		gtk_window_set_default_size (GTK_WINDOW (dialog), 450, -1);
-		gtk_widget_show (dialog);
+		dialog_ptr = &data->properties_dialog;
+	
+		if (*dialog_ptr != NULL)
+		{
+			/* Show already existing dialog */
+			gtk_window_present (GTK_WINDOW (*dialog_ptr));
+		}
+		else
+		{
+			*dialog_ptr = pm_project_create_properties_dialog (
+				plugin->project,
+				GTK_WINDOW (plugin->project->plugin->shell),
+			    plugin->view,
+			    data,                                               
+				selected);
+			if (*dialog_ptr != NULL)
+			{
+				g_object_add_weak_pointer (G_OBJECT (*dialog_ptr), (gpointer *)dialog_ptr);
+			}
+		}
 	}
 
-	return dialog;
+	return selected != NULL;
 }
 
+
+
 /* Group dialog
  *---------------------------------------------------------------------------*/
 
@@ -733,6 +863,7 @@ anjuta_pm_project_new_group (ProjectManagerPlugin *plugin, GtkWindow *parent, Gt
 	GtkBuilder *gui;
 	GtkWidget *dialog, *group_name_entry, *ok_button;
 	GtkWidget *groups_view;
+	GtkTreePath *root;
 	gint response;
 	gboolean finished = FALSE;
 	AnjutaProjectNode *new_group = NULL;
@@ -759,11 +890,14 @@ anjuta_pm_project_new_group (ProjectManagerPlugin *plugin, GtkWindow *parent, Gt
 	else
 		gtk_widget_set_sensitive (ok_button, FALSE);
 
-	setup_nodes_treeview (gbf_project_view_get_model (plugin->view),
-							groups_view,
+	root = gbf_project_model_get_project_root (gbf_project_view_get_model (plugin->view));
+	setup_nodes_treeview (GBF_PROJECT_VIEW (groups_view),
+	                        plugin->view,
+	                      	root,
 							parent_filter_func,
 							GINT_TO_POINTER (ANJUTA_PROJECT_GROUP),
 							selected);
+	gtk_tree_path_free (root);
 	gtk_widget_show (groups_view);
 
 	if (parent)
@@ -874,6 +1008,7 @@ anjuta_pm_project_new_multiple_source (ProjectManagerPlugin *plugin,
 	GtkTreeViewColumn* column_filename;
 	GList* new_sources = NULL;
 	GList* uri_node;
+	GtkTreePath *root;
 
 	g_return_val_if_fail (plugin->project != NULL, NULL);
 
@@ -931,11 +1066,14 @@ anjuta_pm_project_new_multiple_source (ProjectManagerPlugin *plugin,
 	/*project_root = g_file_get_uri (anjuta_project_node_get_file (anjuta_pm_project_get_root (project)));
 	g_object_set_data_full (G_OBJECT (browse_button), "root", project_root, g_free);*/
 
-	setup_nodes_treeview (gbf_project_view_get_model (plugin->view),
-							targets_view,
+	root = gbf_project_model_get_project_root_group (gbf_project_view_get_model (plugin->view));
+	setup_nodes_treeview (GBF_PROJECT_VIEW (targets_view),
+	                        plugin->view,
+	                       	root,
 							parent_filter_func,
 							GINT_TO_POINTER (ANJUTA_PROJECT_SOURCE),
 							default_parent);
+	gtk_tree_path_free (root);
 	gtk_widget_show (targets_view);
 
 	if (top_window) {
@@ -1026,7 +1164,7 @@ anjuta_pm_project_new_multiple_source (ProjectManagerPlugin *plugin,
 			else 
 			{
 				error_dialog (top_window, _("Cannot add source files"),
-						"%s", _("The selected node cannot contains source files."));
+						"%s", _("The selected node cannot contain source files."));
 			}
 			break;
 		}
@@ -1108,6 +1246,7 @@ anjuta_pm_project_new_target (ProjectManagerPlugin *plugin,
 	GtkBuilder *gui;
 	GtkWidget *dialog, *target_name_entry, *ok_button;
 	GtkWidget *target_type_combo, *groups_view;
+	GtkTreePath *root;
 	GtkListStore *types_store;
 	GtkCellRenderer *renderer;
 	gint response;
@@ -1137,11 +1276,14 @@ anjuta_pm_project_new_target (ProjectManagerPlugin *plugin,
 	else
 		gtk_widget_set_sensitive (ok_button, FALSE);
 
-	setup_nodes_treeview (gbf_project_view_get_model (plugin->view),
-							groups_view,
+	root = gbf_project_model_get_project_root_group (gbf_project_view_get_model (plugin->view));
+	setup_nodes_treeview (GBF_PROJECT_VIEW (groups_view),
+	                        plugin->view,
+	                       	root,
 							parent_filter_func,
 							GINT_TO_POINTER (ANJUTA_PROJECT_TARGET),
 							default_group);
+	gtk_tree_path_free (root);
 	gtk_widget_show (groups_view);
 
 	/* setup target types combo box */
@@ -1268,6 +1410,7 @@ anjuta_pm_project_new_module (ProjectManagerPlugin *plugin,
 	GtkWidget *ok_button, *new_button;
 	GtkWidget *targets_view;
 	GtkWidget *modules_view;
+	GtkTreePath *root;
 	gint response;
 	gboolean finished = FALSE;
 	GList* new_modules = NULL;
@@ -1285,17 +1428,23 @@ anjuta_pm_project_new_module (ProjectManagerPlugin *plugin,
 	new_button = GTK_WIDGET (gtk_builder_get_object (gui, "new_package_button"));
 	ok_button = GTK_WIDGET (gtk_builder_get_object (gui, "ok_module_button"));
 
-	setup_nodes_treeview (gbf_project_view_get_model (plugin->view),
-							targets_view,
+	root = gbf_project_model_get_project_root_group (gbf_project_view_get_model (plugin->view));
+	setup_nodes_treeview (GBF_PROJECT_VIEW (targets_view),
+	                        plugin->view,
+	                       	root,
 							parent_filter_func,
 							GINT_TO_POINTER (ANJUTA_PROJECT_MODULE),
 							default_target);
+	gtk_tree_path_free (root);
 	gtk_widget_show (targets_view);
-	setup_nodes_treeview (gbf_project_view_get_model (plugin->view),
-							modules_view,
+	root = gbf_project_model_get_project_root (gbf_project_view_get_model (plugin->view));
+	setup_nodes_treeview (GBF_PROJECT_VIEW (modules_view),
+	                        plugin->view,
+	                        root,
 							module_filter_func,
 							NULL,
 							NULL);
+	gtk_tree_path_free (root);
 	gtk_widget_show (modules_view);
 	module_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (modules_view));
 	gtk_tree_selection_set_mode (module_selection, GTK_SELECTION_MULTIPLE);
