@@ -42,13 +42,13 @@
 #include <libanjuta/interfaces/ianjuta-editor-language.h>
 #include <libanjuta/interfaces/ianjuta-editor-search.h>
 #include <libanjuta/interfaces/ianjuta-editor-hover.h>
+#include <libanjuta/interfaces/ianjuta-editor-glade-signal.h>
 #include <libanjuta/interfaces/ianjuta-provider.h>
 
 #include <gtksourceview/gtksourceview.h>
 #include <gtksourceview/gtksourcelanguage.h>
 #include <gtksourceview/gtksourcelanguagemanager.h>
 #include <gtksourceview/gtksourcebuffer.h>
-#include <gtksourceview/gtksourceiter.h>
 #include <gtksourceview/gtksourcecompletionitem.h>
 
 #include "config.h"
@@ -89,10 +89,12 @@ static gboolean on_sourceview_hover_over (GtkWidget *widget, gint x, gint y,
 
 #define MARKER_PIXMAP_LINEMARKER "anjuta-linemark-16.png"
 #define MARKER_PIXMAP_PROGRAM_COUNTER "anjuta-pcmark-16.png"
+#define MARKER_PIXMAP_MESSAGE "anjuta-message-16.png"
 #define MARKER_PIXMAP_BREAKPOINT_DISABLED "anjuta-breakpoint-disabled-16.png"
 #define MARKER_PIXMAP_BREAKPOINT_ENABLED "anjuta-breakpoint-enabled-16.png"
 #define MARKER_PIXMAP_BOOKMARK "anjuta-bookmark-16.png"
 
+#define MARKER_TOOLTIP_DATA "__tooltip"
 
 /* Keep in sync with IAnjutaMarkableMarker */
 
@@ -100,6 +102,7 @@ static const gchar* marker_types [] =
 {
 	"sv-linemarker",
 	"sv-bookmark",
+	"sv-message",
 	"sv-breakpoint-enabled",
 	"sv-breakpoint-disabled",
 	"sv-program-counter",
@@ -133,65 +136,64 @@ anjuta_message_area_new (const gchar    *text,
 	return GTK_WIDGET (message_area);
 }
 
-/* Create pixmaps for the markers */
-static void sourceview_create_markers(Sourceview* sv)
+static gchar*
+on_marker_tooltip (GtkSourceMarkAttributes* cat, GtkSourceMark* mark, gpointer data)
+{
+	//Sourceview* sv = ANJUTA_SOURCEVIEW (data);
+	gchar* tooltip;
+	tooltip = g_object_get_data (G_OBJECT (mark), MARKER_TOOLTIP_DATA);
+	if (tooltip)
+		return g_strdup (tooltip);
+	else
+		return NULL;
+}
+
+static void 
+sourceview_create_marker_category (Sourceview* sv, const gchar* marker_pixbuf,
+                                   IAnjutaMarkableMarker marker_type)
 {
 	GdkPixbuf * pixbuf;
 	GtkSourceView* view = 	GTK_SOURCE_VIEW(sv->priv->view);
-
-	
-	if ((pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/"MARKER_PIXMAP_BOOKMARK, NULL)))
+	if ((pixbuf = gdk_pixbuf_new_from_file (marker_pixbuf, NULL)))
 	{
-		gtk_source_view_set_mark_category_icon_from_pixbuf (view, 
-		    marker_types[IANJUTA_MARKABLE_BOOKMARK], pixbuf);
-		gtk_source_view_set_mark_category_priority (view, marker_types [IANJUTA_MARKABLE_BOOKMARK],
-											 IANJUTA_MARKABLE_BOOKMARK);
-		g_object_unref (pixbuf);
-	}
-	if ((pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/"MARKER_PIXMAP_BREAKPOINT_DISABLED, NULL)))
-	{
-		gtk_source_view_set_mark_category_icon_from_pixbuf (view, 
-			marker_types[IANJUTA_MARKABLE_BREAKPOINT_DISABLED], pixbuf);
-		gtk_source_view_set_mark_category_priority (view, marker_types [IANJUTA_MARKABLE_BREAKPOINT_DISABLED],
-											 IANJUTA_MARKABLE_BREAKPOINT_DISABLED);
-
-		g_object_unref (pixbuf);
-	}
-	if ((pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/"MARKER_PIXMAP_BREAKPOINT_ENABLED, NULL)))
-	{
-		gtk_source_view_set_mark_category_icon_from_pixbuf (view, 
-			marker_types[IANJUTA_MARKABLE_BREAKPOINT_ENABLED], pixbuf);
-		gtk_source_view_set_mark_category_priority (view, marker_types [IANJUTA_MARKABLE_BREAKPOINT_ENABLED],
-											 IANJUTA_MARKABLE_BREAKPOINT_ENABLED);
-		g_object_unref (pixbuf);
-	}
-	if ((pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/"MARKER_PIXMAP_PROGRAM_COUNTER, NULL)))
-	{
-		gtk_source_view_set_mark_category_icon_from_pixbuf (view, 
-			marker_types[IANJUTA_MARKABLE_PROGRAM_COUNTER], pixbuf);
-		gtk_source_view_set_mark_category_priority (view, marker_types [IANJUTA_MARKABLE_PROGRAM_COUNTER],
-											 IANJUTA_MARKABLE_PROGRAM_COUNTER);
-		g_object_unref (pixbuf);
-	}
-	if ((pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/"MARKER_PIXMAP_LINEMARKER, NULL)))
-	{
-		gtk_source_view_set_mark_category_icon_from_pixbuf (view, 
-			marker_types[IANJUTA_MARKABLE_LINEMARKER], pixbuf);
-		gtk_source_view_set_mark_category_priority (view, marker_types [IANJUTA_MARKABLE_LINEMARKER],
-											 IANJUTA_MARKABLE_LINEMARKER);
+		GtkSourceMarkAttributes* cat = gtk_source_mark_attributes_new ();
+		gtk_source_mark_attributes_set_pixbuf (cat, pixbuf);
+		g_signal_connect (cat, "query-tooltip-text", G_CALLBACK (on_marker_tooltip), sv);
+		gtk_source_view_set_mark_attributes (view, marker_types [marker_type], cat, marker_type);
 		g_object_unref (pixbuf);
 	}
 }
 
-/* Create tags for highlighting */
-static void sourceview_create_highligth_indic(Sourceview* sv)
+/* Create pixmaps for the markers */
+static void sourceview_create_markers(Sourceview* sv)
 {
+	sourceview_create_marker_category (sv, PACKAGE_PIXMAPS_DIR"/"MARKER_PIXMAP_BOOKMARK,
+	                                   IANJUTA_MARKABLE_BOOKMARK);
+	sourceview_create_marker_category (sv, PACKAGE_PIXMAPS_DIR"/"MARKER_PIXMAP_BREAKPOINT_DISABLED,
+	                                   IANJUTA_MARKABLE_BREAKPOINT_DISABLED);
+	sourceview_create_marker_category (sv, PACKAGE_PIXMAPS_DIR"/"MARKER_PIXMAP_BREAKPOINT_ENABLED,
+	                                   IANJUTA_MARKABLE_BREAKPOINT_ENABLED);
+	sourceview_create_marker_category (sv, PACKAGE_PIXMAPS_DIR"/"MARKER_PIXMAP_PROGRAM_COUNTER,
+	                                   IANJUTA_MARKABLE_PROGRAM_COUNTER);
+	sourceview_create_marker_category (sv, PACKAGE_PIXMAPS_DIR"/"MARKER_PIXMAP_LINEMARKER,
+	                                   IANJUTA_MARKABLE_LINEMARKER);
+	sourceview_create_marker_category (sv, PACKAGE_PIXMAPS_DIR"/"MARKER_PIXMAP_MESSAGE,
+	                                   IANJUTA_MARKABLE_MESSAGE);
+}
+
+#define PREF_COLOR_ERROR "msgman-color-error"
+#define PREF_COLOR_WARNING "msgman-color-warning"
+
+
+/* Create tags for highlighting */
+static void sourceview_create_highlight_indic(Sourceview* sv)
+{	
 	char* error_color =
-		anjuta_preferences_get (anjuta_preferences_default(),
-		                        "messages.color.error");
+		g_settings_get_string (sv->priv->msgman_settings,
+		                       PREF_COLOR_ERROR);
 	char* warning_color =
-		anjuta_preferences_get (anjuta_preferences_default (),
-		                        "messages.color.warning");
+		g_settings_get_string (sv->priv->msgman_settings,
+		                       PREF_COLOR_WARNING);
 	sv->priv->important_indic = 
 		gtk_text_buffer_create_tag (GTK_TEXT_BUFFER(sv->priv->document),
 									IMPORTANT_INDIC,
@@ -760,11 +762,16 @@ sourceview_instance_init(Sourceview* sv)
 
 	g_object_set (G_OBJECT (sv->priv->view), "has-tooltip", TRUE, NULL);
 	gtk_source_view_set_smart_home_end(GTK_SOURCE_VIEW(sv->priv->view), FALSE);
+
+	/* Apply Preferences */
+	sourceview_prefs_init(sv);
+
 	
 	/* Create Markers */
 	sourceview_create_markers(sv);
+
 	/* Create Higlight Tag */
-	sourceview_create_highligth_indic(sv);
+	sourceview_create_highlight_indic(sv);
 }
 
 static void
@@ -825,9 +832,6 @@ sourceview_new(GFile* file, const gchar* filename, AnjutaPlugin* plugin)
 	
 	Sourceview *sv = ANJUTA_SOURCEVIEW(g_object_new(ANJUTA_TYPE_SOURCEVIEW, NULL));
 	
-	/* Apply Preferences */
-	sv->priv->prefs = anjuta_preferences_default();
-	sourceview_prefs_init(sv);
 	sv->priv->plugin = plugin;
 	
 	/* Add View */
@@ -1144,9 +1148,17 @@ static void ieditor_insert(IAnjutaEditor *editor, IAnjutaIterable* icell,
 	GtkTextIter iter;
 	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
 	sourceview_cell_get_iter (cell, &iter);
+
+	/* Avoid processing text that is inserted programatically */
+	g_signal_handlers_block_by_func (sv->priv->document,
+	                                 on_insert_text,
+	                                 sv);
 	
 	gtk_text_buffer_insert(GTK_TEXT_BUFFER(sv->priv->document),
 						   &iter, text, length);
+	g_signal_handlers_unblock_by_func (sv->priv->document,
+	                                   on_insert_text,
+	                                   sv);
 }
 
 /* Append text to buffer */
@@ -1158,8 +1170,16 @@ static void ieditor_append(IAnjutaEditor *editor, const gchar* text,
 
 	gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(sv->priv->document),
 									   &iter);
+
+	/* Avoid processing text that is inserted programatically */
+	g_signal_handlers_block_by_func (sv->priv->document,
+	                                 on_insert_text,
+	                                 sv);
 	gtk_text_buffer_insert(GTK_TEXT_BUFFER(sv->priv->document),
 						   &iter, text, length);
+	g_signal_handlers_unblock_by_func (sv->priv->document,
+	                                   on_insert_text,
+	                                   sv);
 }
 
 static void ieditor_erase(IAnjutaEditor* editor, IAnjutaIterable* istart_cell, 
@@ -1732,6 +1752,7 @@ typedef struct
 	gint location;
 	gint handle;
 	guint source;
+	gchar* tooltip;
 	Sourceview* sv;
 } SVMark;
 
@@ -1744,6 +1765,7 @@ static gboolean mark_real (gpointer data)
 	const gchar* category;
 	gint location = svmark->location;
 	gint marker_count = svmark->handle;
+	gchar* tooltip = svmark->tooltip;
 	IAnjutaMarkableMarker marker = svmark->marker;
 	gchar* name;
 	
@@ -1762,6 +1784,8 @@ static gboolean mark_real (gpointer data)
 	
 	source_mark = gtk_source_buffer_create_source_mark(GTK_SOURCE_BUFFER(sv->priv->document), 
 												name, category, &iter);
+	g_object_set_data_full (G_OBJECT (source_mark), MARKER_TOOLTIP_DATA, tooltip,
+	                        (GDestroyNotify) g_free);
 	
 	g_source_remove (svmark->source);
 	
@@ -1772,7 +1796,7 @@ static gboolean mark_real (gpointer data)
 
 static gint
 imark_mark(IAnjutaMarkable* mark, gint location, IAnjutaMarkableMarker marker,
-		   GError **e)
+           const gchar* tooltip, GError **e)
 {
 	Sourceview* sv = ANJUTA_SOURCEVIEW(mark);
 	SVMark* svmark = g_slice_new0 (SVMark);
@@ -1792,6 +1816,7 @@ imark_mark(IAnjutaMarkable* mark, gint location, IAnjutaMarkableMarker marker,
 	svmark->location = location;
 	svmark->handle = marker_count;
 	svmark->marker = marker;
+	svmark->tooltip = tooltip ? g_strdup (tooltip) : NULL;
 	svmark->source = g_idle_add (mark_real, svmark);
 	
 	sv->priv->idle_sources = g_slist_prepend (sv->priv->idle_sources,
@@ -2005,7 +2030,9 @@ autodetect_language (Sourceview* sv)
 	
 	language = gtk_source_language_manager_guess_language (gtk_source_language_manager_get_default (), filename, io_mime_type);
 	if (!language)
+	{
 		goto out;
+	}
 	
 	detected_language = gtk_source_language_get_id (language);	
 	
@@ -2285,7 +2312,7 @@ isearch_forward (IAnjutaEditorSearch* isearch,
 	
 	GtkTextIter result_start, result_end;
 	
-	GtkSourceSearchFlags flags = 0;
+	GtkTextSearchFlags flags = 0;
 
 	sourceview_cell_get_iter (start, &start_iter);
 	sourceview_cell_get_iter (end, &end_iter);
@@ -2293,11 +2320,11 @@ isearch_forward (IAnjutaEditorSearch* isearch,
 	
 	if (!case_sensitive)
 	{
-		flags = GTK_SOURCE_SEARCH_CASE_INSENSITIVE;
+		flags = GTK_TEXT_SEARCH_CASE_INSENSITIVE;
 	}
 	
 	gboolean result = 
-		gtk_source_iter_forward_search (&start_iter, search, flags, &result_start, &result_end,
+		gtk_text_iter_forward_search (&start_iter, search, flags, &result_start, &result_end,
 									&end_iter);
 
 	if (result)
@@ -2336,7 +2363,7 @@ isearch_backward (IAnjutaEditorSearch* isearch,
 	GtkTextIter end_iter;	
 	GtkTextIter result_start, result_end;
 	
-	GtkSourceSearchFlags flags = 0;
+	GtkTextSearchFlags flags = 0;
 	
 	sourceview_cell_get_iter (start, &start_iter);
 	sourceview_cell_get_iter (end, &end_iter);
@@ -2344,11 +2371,11 @@ isearch_backward (IAnjutaEditorSearch* isearch,
 	
 	if (!case_sensitive)
 	{
-		flags = GTK_SOURCE_SEARCH_CASE_INSENSITIVE;
+		flags = GTK_TEXT_SEARCH_CASE_INSENSITIVE;
 	}
 	
 	gboolean result = 
-		gtk_source_iter_backward_search (&start_iter, search, flags, &result_start, &result_end,
+		gtk_text_iter_backward_search (&start_iter, search, flags, &result_start, &result_end,
 									&end_iter);
 
 	if (result)
@@ -2449,6 +2476,12 @@ ihover_iface_init(IAnjutaEditorHoverIface* iface)
 	iface->display = ihover_display;
 }
 
+static void
+iglade_iface_init(IAnjutaEditorGladeSignalIface* iface)
+{
+	/* only signals */
+}
+
 ANJUTA_TYPE_BEGIN(Sourceview, sourceview, GTK_TYPE_VBOX);
 ANJUTA_TYPE_ADD_INTERFACE(idocument, IANJUTA_TYPE_DOCUMENT);
 ANJUTA_TYPE_ADD_INTERFACE(ifile, IANJUTA_TYPE_FILE);
@@ -2464,4 +2497,5 @@ ANJUTA_TYPE_ADD_INTERFACE(iprint, IANJUTA_TYPE_PRINT);
 ANJUTA_TYPE_ADD_INTERFACE(ilanguage, IANJUTA_TYPE_EDITOR_LANGUAGE);
 ANJUTA_TYPE_ADD_INTERFACE(isearch, IANJUTA_TYPE_EDITOR_SEARCH);
 ANJUTA_TYPE_ADD_INTERFACE(ihover, IANJUTA_TYPE_EDITOR_HOVER);
+ANJUTA_TYPE_ADD_INTERFACE(iglade, IANJUTA_TYPE_EDITOR_GLADE_SIGNAL);
 ANJUTA_TYPE_END;
